@@ -70,6 +70,7 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
   saveWidgetsCheckBox->setChecked(m_options->saveWidgets);
   iconTextCheckBox->setChecked(m_options->iconText);
   wrapLinesCheckBox->setChecked(m_options->wrapLines);
+  autoCompleteCheckBox->setChecked(m_options->autoComplete);
   widgetsCheckBox->setChecked(m_options->enableWidgets);
   channelComboBox->setCurrentIndex(m_options->useInvalue ? 0: 1);
   showWidgetsOnRunCheckBox->setChecked(m_options->showWidgetsOnRun);
@@ -77,6 +78,8 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
   enableFLTKCheckBox->setChecked(m_options->enableFLTK);
   terminalFLTKCheckBox->setChecked(m_options->terminalFLTK);
   terminalFLTKCheckBox->setEnabled(m_options->enableFLTK);
+  scrollbarsCheckBox->setChecked(m_options->scrollbars);
+  newFormatCheckBox->setChecked(m_options->newformat);
 
   if (m_options->useAPI)
     ApiRadioButton->setChecked(true);
@@ -85,6 +88,13 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
 
   threadCheckBox->setChecked(m_options->thread);
   threadCheckBox->setEnabled(ApiRadioButton->isChecked());
+  keyRepeatCheckBox->setChecked(m_options->keyRepeat);
+  debugLiveEventsCheckBox->setChecked(m_options->debugLiveEvents);
+  int bufferIndex = consoleBufferComboBox->findText(QString::number(m_options->consoleBufferSize));
+  if (bufferIndex < 0) {
+    bufferIndex = consoleBufferComboBox->count() - 1;
+  }
+  consoleBufferComboBox->setCurrentIndex(bufferIndex);
 
   BufferSizeLineEdit->setText(QString::number(m_options->bufferSize));
   BufferSizeCheckBox->setChecked(m_options->bufferSizeActive);
@@ -117,6 +127,7 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
   RtMidiModuleComboBox->setCurrentIndex(m_options->rtMidiModule);
   RtMidiInputLineEdit->setText(m_options->rtMidiInputDevice);
   RtMidiOutputLineEdit->setText(m_options->rtMidiOutputDevice);
+  simultaneousCheckBox->setChecked(m_options->simultaneousRun);
 
   sampleFormatComboBox->setCurrentIndex(m_options->sampleFormat);
 
@@ -139,6 +150,9 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
   defaultCsdCheckBox->setChecked(m_options->defaultCsdActive);
   defaultCsdLineEdit->setText(m_options->defaultCsd);
   defaultCsdLineEdit->setEnabled(m_options->defaultCsdActive);
+  favoriteLineEdit->setText(m_options->favoriteDir);
+  pythonDirLineEdit->setText(m_options->pythonDir);
+  logFileLineEdit->setText(m_options->logFile);
 
   TerminalLineEdit->setText(m_options->terminal);
   browserLineEdit->setText(m_options->browser);
@@ -157,13 +171,16 @@ ConfigDialog::ConfigDialog(qutecsound *parent, Options *options)
   connect(sfdirToolButton, SIGNAL(clicked()), this, SLOT(browseSfdir()));
   connect(incdirToolButton, SIGNAL(clicked()), this, SLOT(browseIncdir()));
   connect(defaultCsdToolButton, SIGNAL(clicked()), this, SLOT(browseDefaultCsd()));
+  connect(favoriteToolButton, SIGNAL(clicked()), this, SLOT(browseFavorite()));
   connect(terminalToolButton, SIGNAL(clicked()), this, SLOT(browseTerminal()));
   connect(browserToolButton, SIGNAL(clicked()), this, SLOT(browseBrowser()));
   connect(dotToolButton, SIGNAL(clicked()), this, SLOT(browseDot()));
   connect(waveEditorToolButton, SIGNAL(clicked()), this, SLOT(browseWaveEditor()));
   connect(wavePlayerToolButton, SIGNAL(clicked()), this, SLOT(browseWavePlayer()));
   connect(pdfViewerToolButton, SIGNAL(clicked()), this, SLOT(browsePdfViewer()));
-  connect(this, SIGNAL(changeFont()), parent, SLOT(changeFont()));
+  connect(pythonDirToolButton, SIGNAL(clicked()), this, SLOT(browsePythonDir()));
+  connect(logFileToolButton, SIGNAL(clicked()), this, SLOT(browseLogFile()));
+//  connect(this, SIGNAL(changeFont()), parent, SLOT(changeFont()));
   connect(audioInputToolButton, SIGNAL(released()), this, SLOT(selectAudioInput()));
   connect(audioOutputToolButton, SIGNAL(released()), this, SLOT(selectAudioOutput()));
   connect(midiInputToolButton, SIGNAL(released()), this, SLOT(selectMidiInput()));
@@ -187,7 +204,7 @@ void ConfigDialog::warnOpcodeDir(bool on)
 {
   if (on) {
     QMessageBox::warning(this, "QuteCsound",
-                         tr("Please note that OPCODEDIR will only be set when running in an external shell. It won't be set when using Run or Render.")
+                         tr("Please note that OPCODEDIR will overwrite current settings so you will need to restart QuteCsound to return to default.")
                         );
   }
 }
@@ -213,15 +230,23 @@ void ConfigDialog::accept()
   m_options->saveWidgets = saveWidgetsCheckBox->isChecked();
   m_options->iconText = iconTextCheckBox->isChecked();
   m_options->wrapLines = wrapLinesCheckBox->isChecked();
+  m_options->autoComplete = autoCompleteCheckBox->isChecked();
   m_options->enableWidgets = widgetsCheckBox->isChecked();
   m_options->useInvalue = channelComboBox->currentIndex() == 0;
   m_options->showWidgetsOnRun = showWidgetsOnRunCheckBox->isChecked();
   m_options->showTooltips = showTooltipsCheckBox->isChecked();
   m_options->enableFLTK = enableFLTKCheckBox->isChecked();
   m_options->terminalFLTK = terminalFLTKCheckBox->isChecked();
+  m_options->scrollbars = scrollbarsCheckBox->isChecked();
+  m_options->newformat = newFormatCheckBox->isChecked();
 
   m_options->useAPI = ApiRadioButton->isChecked();
   m_options->thread = threadCheckBox->isChecked();
+  m_options->keyRepeat = keyRepeatCheckBox->isChecked();
+  m_options->debugLiveEvents = debugLiveEventsCheckBox->isChecked();
+  m_options->consoleBufferSize = consoleBufferComboBox->itemText(consoleBufferComboBox->currentIndex()).toInt();
+  if (m_options->consoleBufferSize < 0)
+    m_options->consoleBufferSize = 0;
   m_options->bufferSize = BufferSizeLineEdit->text().toInt();
   m_options->bufferSizeActive = BufferSizeCheckBox->isChecked();
   m_options->HwBufferSize = HwBufferSizeLineEdit->text().toInt();
@@ -248,6 +273,7 @@ void ConfigDialog::accept()
   m_options->rtMidiModule = RtMidiModuleComboBox->currentIndex();
   m_options->rtMidiInputDevice = RtMidiInputLineEdit->text();
   m_options->rtMidiOutputDevice = RtMidiOutputLineEdit->text();
+  m_options->simultaneousRun = simultaneousCheckBox->isChecked();
 
   m_options->sampleFormat = sampleFormatComboBox->currentIndex();
 
@@ -264,6 +290,9 @@ void ConfigDialog::accept()
   m_options->incdir = IncdirLineEdit->text();
   m_options->defaultCsdActive = defaultCsdCheckBox->isChecked();
   m_options->defaultCsd = defaultCsdLineEdit->text();
+  m_options->favoriteDir = favoriteLineEdit->text();
+  m_options->pythonDir = pythonDirLineEdit->text();
+  m_options->logFile = logFileLineEdit->text();
 
   m_options->terminal = TerminalLineEdit->text();
   m_options->browser = browserLineEdit->text();
@@ -273,7 +302,7 @@ void ConfigDialog::accept()
   m_options->pdfviewer = pdfViewerLineEdit->text();
   m_options->language = languageComboBox->currentIndex();
 
-  emit(changeFont());
+//  emit(changeFont());
   QDialog::accept();
 }
 
@@ -336,6 +365,12 @@ void ConfigDialog::browseDefaultCsd()
   defaultCsdLineEdit->setText(m_options->defaultCsd);
 }
 
+void ConfigDialog::browseFavorite()
+{
+  browseDir(m_options->favoriteDir);
+  favoriteLineEdit->setText(m_options->favoriteDir);
+}
+
 void ConfigDialog::browseTerminal()
 {
   browseFile(m_options->terminal);
@@ -372,6 +407,18 @@ void ConfigDialog::browsePdfViewer()
   pdfViewerLineEdit->setText(m_options->pdfviewer);
 }
 
+void ConfigDialog::browsePythonDir()
+{
+  browseDir(m_options->pythonDir);
+  pythonDirLineEdit->setText(m_options->pythonDir);
+}
+
+void ConfigDialog::browseLogFile()
+{
+  browseSaveFile(m_options->logFile);
+  logFileLineEdit->setText(m_options->logFile);
+}
+
 void ConfigDialog::selectAudioInput()
 {
   QList<QPair<QString, QString> > deviceList = getAudioInputDevices();
@@ -430,7 +477,7 @@ void ConfigDialog::selectMidiInput()
   QVector<QAction*> actions;
 
   QPair<QString, QString> device;
-  device.first = "none";
+  device.first = "Disabled";
   device.second = "";
   deviceList.prepend(device);
 
@@ -440,12 +487,6 @@ void ConfigDialog::selectMidiInput()
     deviceList.prepend(device);
   }
 
-  if (module == "virtual") {
-    device.first = "enabled";
-    device.second = "0";
-    deviceList.prepend(device);
-  }
-
   for (int i = 0; i < deviceList.size(); i++) {
     QAction* action =  menu.addAction(deviceList[i].first + " (" + deviceList[i].second +")");
     actions.append(action);
@@ -501,45 +542,42 @@ void ConfigDialog::browseDir(QString &destination)
 
 QList<QPair<QString, QString> > ConfigDialog::getMidiInputDevices()
 {
-  // Adapted from blue by Steven Yi
+  // based on code by Steven Yi
   QList<QPair<QString, QString> > deviceList;
   QString module = _configlists.rtMidiNames[RtMidiModuleComboBox->currentIndex()];
   if (module == "none") {
     return deviceList;
   }
   if (module == "alsa") {
-    // This commented section uses the wrong file to query devices. maybe this used to work? 
-//     QFile f("/proc/asound/seq/clients");
-//     f.open(QIODevice::ReadOnly | QIODevice::Text);
-//     QString values = QString(f.readAll());
-//     QStringList st = values.split("\n");
-//     QString line;
-//     for (int i = 0; i < st.size(); i++){
-//       line = st[i].trimmed();
-//       if (line.startsWith("Client") && line.indexOf(":") >= 0) {
-//         QString clientNum = QString::number(line.mid(6,line.indexOf(":") - 6).trimmed().toInt());
-//         while (i < st.size() - 2) {
-//           i++;
-// 
-//           QString tempLine = st[i].trimmed();
-//           if (tempLine.startsWith("Port")) {
-//             QString capabilities = tempLine.mid(tempLine.lastIndexOf("("));
-//             if (capabilities.indexOf("R") >= 0) {
-//               QStringList portParts = tempLine.split("\"");
-//               int portNum = tempLine.mid(6, tempLine.indexOf(":") - 6).trimmed().toInt();
-//               QPair<QString, QString> device;
-//               device.first = portParts[1];
-//               device.second = "hw:" + clientNum + "," + QString::number(portNum);
-//               deviceList.append(device);
-//             }
-//           }
-//           else if (tempLine.startsWith("Client")) {
-//             i--;
-//             break;
-//           }
-//         }
-//       }
-//     }
+    QProcess amidi;
+    amidi.start("amidi", QStringList() << "-l");
+    if (!amidi.waitForFinished())
+      return deviceList;
+
+    QByteArray result = amidi.readAllStandardOutput();
+    QString values = QString(result);
+    QStringList st = values.split("\n");
+    st.takeFirst(); // Remove first column lines
+    for (int i = 0; i < st.size(); i++){
+      QStringList parts = st[i].split(" ", QString::SkipEmptyParts);
+      if (parts.size() > 0 && parts[0].contains("I")) {
+        QPair<QString, QString> device;
+        device.second = parts[1]; // Devce name
+        parts.takeFirst(); // Remove IO flags
+        device.first = parts.join(" ") ; // Full name with description
+        deviceList.append(device);
+      }
+    }
+    QPair<QString, QString> device;
+    device.first = "All available devices "; // Full name with description
+    device.second = "a"; // Devce name
+    deviceList.append(device);
+  }
+  else if (module == "virtual") {
+    QPair<QString, QString> device;
+    device.first = tr("Enabled", "Virtual MIDI keyboard Enabled");
+    device.second = "0";
+    deviceList.append(device);
   }
   else { // if not alsa (i.e. winmm or portmidi)
     QFile file(":/test.csd");
@@ -547,8 +585,8 @@ QList<QPair<QString, QString> > ConfigDialog::getMidiInputDevices()
       return deviceList;
     QString jackCSD = QString(file.readAll());
     QString tempText = jackCSD;
-    tempText.replace("$SR", "1000");
-    QTemporaryFile tempFile(QDir::tempPath() + "/" + "testcsdQuteCsoundXXXXXX.csd");
+    tempText.replace("$SR", "441000");
+    QTemporaryFile tempFile(QDir::tempPath() + QDir::separator() + "testcsdQuteCsoundXXXXXX.csd");
     tempFile.open();
     QTextStream out(&tempFile);
     out << tempText;
@@ -557,7 +595,7 @@ QList<QPair<QString, QString> > ConfigDialog::getMidiInputDevices()
 
     QStringList flags;
 //     QString rtAudioFlag = "-+rtaudio=" + module;
-    flags << "-+msg_color=false"/* << rtAudioFlag*/ << "-odac"  << "-M999" << tempFile.fileName();
+    flags << "-+msg_color=false"/* << rtAudioFlag*/ << "-otest"  << "-n"  << "-M999" << tempFile.fileName();
     QStringList messages = m_parent->runCsoundInternally(flags);
 
     QString startText, endText;
@@ -604,46 +642,37 @@ QList<QPair<QString, QString> > ConfigDialog::getMidiOutputDevices()
     return deviceList;
   }
   if (module == "alsa") {
-    QFile f("/proc/asound/seq/clients");
-    f.open(QIODevice::ReadOnly | QIODevice::Text);
-    QString values = QString(f.readAll());
-    QStringList st = values.split("\n");
-    QString line;
-    for (int i = 0; i < st.size(); i++){
-      line = st[i].trimmed();
-      if (line.startsWith("Client") && line.indexOf(":") >= 0) {
-        QString clientNum = QString::number(line.mid(6,line.indexOf(":") - 6).trimmed().toInt());
-        while (i < st.size() - 2) {
-          i++;
+    QProcess amidi;
+    amidi.start("amidi", QStringList() << "-l");
+    if (!amidi.waitForFinished())
+      return deviceList;
 
-          QString tempLine = st[i].trimmed();
-          if (tempLine.startsWith("Port")) {
-            QString capabilities = tempLine.mid(tempLine.lastIndexOf("("));
-            if (capabilities.indexOf("W") >= 0) {
-              QStringList portParts = tempLine.split("\"");
-              int portNum = tempLine.mid(6, tempLine.indexOf(":") - 6).trimmed().toInt();
-              QPair<QString, QString> device;
-              device.first = portParts[1];
-              device.second = "hw:" + clientNum + "," + QString::number(portNum);
-              deviceList.append(device);
-            }
-          }
-          else if (tempLine.startsWith("Client")) {
-            i--;
-            break;
-          }
-        }
+    QByteArray result = amidi.readAllStandardOutput();
+    QString values = QString(result);
+    QStringList st = values.split("\n");
+    st.takeFirst(); // Remove first column lines
+    for (int i = 0; i < st.size(); i++){
+      QStringList parts = st[i].split(" ", QString::SkipEmptyParts);
+      if (parts.size() > 0 && parts[0].contains("O")) {
+        QPair<QString, QString> device;
+        device.second = parts[1]; // Devce name
+        parts.takeFirst(); // Remove IO flags
+        device.first = parts.join(" ") ; // Full name with description
+        deviceList.append(device);
       }
     }
   }
+  else if (module == "virtual") {
+    // do nothing
+  }
   else { // if not alsa (i.e. winmm or portmidi)
     QFile file(":/test.csd");
     if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
       return deviceList;
     QString jackCSD = QString(file.readAll());
     QString tempText = jackCSD;
-    tempText.replace("$SR", "1000");
-    QTemporaryFile tempFile(QDir::tempPath() + "/" + "testcsdQuteCsoundXXXXXX.csd");
+    tempText.replace("$SR", "441000");
+    QTemporaryFile tempFile(QDir::tempPath() + QDir::separator() + "testcsdQuteCsoundXXXXXX.csd");
     tempFile.open();
     QTextStream out(&tempFile);
     out << tempText;
@@ -651,7 +680,7 @@ QList<QPair<QString, QString> > ConfigDialog::getMidiOutputDevices()
     tempFile.open();
 
     QStringList flags;
-    flags << "-+msg_color=false" << "-odac"  << "-Q999" << tempFile.fileName();
+    flags << "-+msg_color=false" << "-otest"  << "-n"   << "-Q999" << tempFile.fileName();
     QStringList messages = m_parent->runCsoundInternally(flags);
 
     QString startText, endText;
@@ -692,7 +721,7 @@ QList<QPair<QString, QString> > ConfigDialog::getMidiOutputDevices()
 
 QList<QPair<QString, QString> > ConfigDialog::getAudioInputDevices()
 {
-  qDebug("qutecsound::getAudioInputDevices()");
+//  qDebug("qutecsound::getAudioInputDevices()");
   QList<QPair<QString, QString> > deviceList;
   QString module = _configlists.rtAudioNames[RtModuleComboBox->currentIndex()];
   if (module == "none") {
@@ -718,14 +747,14 @@ QList<QPair<QString, QString> > ConfigDialog::getAudioInputDevices()
       }
     }
   }
-  else if (module=="jack") {
+  else if (module == "jack") {
     QFile file(":/test.csd");
     if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
       return deviceList;
     QString jackCSD = QString(file.readAll());
     QString tempText = jackCSD;
     tempText.replace("$SR", "1000");
-    QTemporaryFile tempFile(QDir::tempPath() + "/" + "testcsdQuteCsoundXXXXXX.csd");
+    QTemporaryFile tempFile(QDir::tempPath() + QDir::separator() + "testcsdQuteCsoundXXXXXX.csd");
     tempFile.open();
     QTextStream out(&tempFile);
     out << tempText;
@@ -733,15 +762,24 @@ QList<QPair<QString, QString> > ConfigDialog::getAudioInputDevices()
     tempFile.open();
 
     QStringList flags;
+    QString previousLine = "";
     // -odac is needed otherwise csound segfaults
     flags << "-+msg_color=false" << "-+rtaudio=jack" << "-iadc:xxx" << "-odac:xxx" << "-B2048" <<  tempFile.fileName();
     QStringList messages = m_parent->runCsoundInternally(flags);
 
     QString sr = "";
-    foreach (QString line, messages) {
+    foreach (QString line, messages) { // Need to run again if sample rate does not match...
       if (line.indexOf("does not match JACK sample rate") >= 0) {
         sr = line.mid(line.lastIndexOf(" ") + 1);
       }
+      if (line.endsWith("channel)\n") || line.endsWith("channels)\n")) {
+        QStringList parts = previousLine.split("\"");
+        QPair<QString, QString> device;
+        device.first = parts[1];
+        device.second = "adc:" + parts[1];
+        deviceList.append(device);
+      }
+      previousLine = line;
     }
     if (sr == "") {
       return deviceList;
@@ -755,9 +793,7 @@ QList<QPair<QString, QString> > ConfigDialog::getAudioInputDevices()
 
     messages = m_parent->runCsoundInternally(flags); // run with same flags as before
 
-    QString previousLine = "";
     foreach (QString line, messages) {
-      qDebug("jack %s----",line.toStdString().c_str());
       if (line.endsWith("channel)\n") || line.endsWith("channels)\n")) {
         QStringList parts = previousLine.split("\"");
         QPair<QString, QString> device;
@@ -775,7 +811,7 @@ QList<QPair<QString, QString> > ConfigDialog::getAudioInputDevices()
     QString jackCSD = QString(file.readAll());
     QString tempText = jackCSD;
     tempText.replace("$SR", "1000");
-    QTemporaryFile tempFile(QDir::tempPath() + "/" + "testcsdQuteCsoundXXXXXX.csd");
+    QTemporaryFile tempFile(QDir::tempPath() + QDir::separator() + "testcsdQuteCsoundXXXXXX.csd");
     tempFile.open();
     QTextStream out(&tempFile);
     out << tempText;
@@ -840,7 +876,7 @@ QList<QPair<QString, QString> > ConfigDialog::getAudioInputDevices()
 
 QList<QPair<QString, QString> > ConfigDialog::getAudioOutputDevices()
 {
-  qDebug("qutecsound::getAudioOutputDevices()");
+//  qDebug("qutecsound::getAudioOutputDevices()");
   QList<QPair<QString, QString> > deviceList;
   QString module = _configlists.rtAudioNames[RtModuleComboBox->currentIndex()];
   if (module == "none") {
@@ -873,7 +909,7 @@ QList<QPair<QString, QString> > ConfigDialog::getAudioOutputDevices()
     QString jackCSD = QString(file.readAll());
     QString tempText = jackCSD;
     tempText.replace("$SR", "1000");
-    QTemporaryFile tempFile(QDir::tempPath() + "/" + "testcsdQuteCsoundXXXXXX.csd");
+    QTemporaryFile tempFile(QDir::tempPath() + QDir::separator() + "testcsdQuteCsoundXXXXXX.csd");
     tempFile.open();
     QTextStream out(&tempFile);
     out << tempText;
@@ -881,6 +917,7 @@ QList<QPair<QString, QString> > ConfigDialog::getAudioOutputDevices()
     tempFile.open();
 
     QStringList flags;
+    QString previousLine = "";
     flags << "-+msg_color=false" << "-+rtaudio=jack" << "-odac:xxx" << "-B2048" <<  tempFile.fileName();
     QStringList messages = m_parent->runCsoundInternally(flags);
 
@@ -889,6 +926,14 @@ QList<QPair<QString, QString> > ConfigDialog::getAudioOutputDevices()
       if (line.indexOf("does not match JACK sample rate") >= 0) {
         sr = line.mid(line.lastIndexOf(" ") + 1);
       }
+      if (line.endsWith("channel)\n") || line.endsWith("channels)\n")) {
+        QStringList parts = previousLine.split("\"");
+        QPair<QString, QString> device;
+        device.first = parts[1];
+        device.second = "dac:" + parts[1];
+        deviceList.append(device);
+      }
+      previousLine = line;
     }
     if (sr == "") {
       return deviceList;
@@ -902,7 +947,6 @@ QList<QPair<QString, QString> > ConfigDialog::getAudioOutputDevices()
 
     messages = m_parent->runCsoundInternally(flags); // run with same flags as before
 
-    QString previousLine = "";
     foreach (QString line, messages) {
       if (line.endsWith("channel)\n") || line.endsWith("channels)\n")) {
         QStringList parts = previousLine.split("\"");
@@ -921,7 +965,7 @@ QList<QPair<QString, QString> > ConfigDialog::getAudioOutputDevices()
     QString jackCSD = QString(file.readAll());
     QString tempText = jackCSD;
     tempText.replace("$SR", "1000");
-    QTemporaryFile tempFile(QDir::tempPath() + "/" + "testcsdQuteCsoundXXXXXX.csd");
+    QTemporaryFile tempFile(QDir::tempPath() + QDir::separator() + "testcsdQuteCsoundXXXXXX.csd");
     tempFile.open();
     QTextStream out(&tempFile);
     out << tempText;
