@@ -34,7 +34,10 @@ class QuteConsole : public QuteWidget
 
     ~QuteConsole();
 
+    virtual void loadFromXml(QString xmlText);
     virtual QString getWidgetLine();
+    virtual QString getWidgetXmlText();
+    virtual QString getWidgetType();
     virtual void setWidgetGeometry(int x,int y,int width,int height);
 //     void popUpMenu(QPoint pos);
 
