@@ -27,8 +27,31 @@ QuteDummy::QuteDummy(QWidget *parent) : QuteWidget(parent)
   m_widget = new QFrame(this);
   m_widget->setAutoFillBackground (true);
   m_widget->setPalette(QPalette(Qt::gray));
+  m_widget->setMouseTracking(true); // Necessary to pass mouse tracking to widget panel for _MouseX channels
 }
 
 QuteDummy::~QuteDummy()
 {
 }
+
+void QuteDummy::loadFromXml(QString xmlText)
+{
+  qDebug() << "loadFromXml not implemented for this widget yet";
+}
+
+
+QString QuteDummy::getWidgetXmlText()
+{
+  // Just an empty place holder
+  QXmlStreamWriter s(&xmlText);
+  createXmlWriter(s);
+
+//   s.writeTextElement("randomizable", "");
+  s.writeEndElement();
+  return xmlText;
+}
+
+QString QuteDummy::getWidgetType()
+{
+  return QString("BSBDummy");
+}
