@@ -26,13 +26,13 @@
 // Font point sizes equivalent for html
 // This seems necessary since qt rich text
 // only takes these values for font size
-#define QUTE_XXSMALL 8
-#define QUTE_XSMALL 10
-#define QUTE_SMALL 12
-#define QUTE_MEDIUM 16
-#define QUTE_LARGE 20
-#define QUTE_XLARGE 24
-#define QUTE_XXLARGE 28
+#define QCS_XXSMALL 8
+#define QCS_XSMALL 10
+#define QCS_SMALL 12
+#define QCS_MEDIUM 16
+#define QCS_LARGE 20
+#define QCS_XLARGE 24
+#define QCS_XXLARGE 28
 
 //number of decimal places shown on the widget
 #define QUTESLIDER_PRECISION 4
@@ -45,6 +45,10 @@ QuteText::QuteText(QWidget *parent) : QuteWidget(parent)
   static_cast<QLabel*>(m_widget)->setMargin (5);
   static_cast<QLabel*>(m_widget)->setTextFormat(Qt::RichText);
   m_widget->setContextMenuPolicy(Qt::NoContextMenu);
+  m_widget->setMouseTracking(true); // Necessary to pass mouse tracking to widget panel for _MouseX channels
+  setMouseTracking(true);
+//  canFocus(true);
+
 //   connect(static_cast<QLabel*>(m_widget), SIGNAL(popUpMenu(QPoint)), this, SLOT(popUpMenu(QPoint)));
 }
 
@@ -52,6 +56,11 @@ QuteText::~QuteText()
 {
 }
 
+void QuteText::loadFromXml(QString xmlText)
+{
+  qDebug() << "loadFromXml not implemented for this widget yet";
+}
+
 double QuteText::getValue()
 {
   return m_value;
@@ -149,17 +158,17 @@ void QuteText::setText(QString text)
 {
   int size;
   m_text = text.replace("\u00AC", "\n");
-  if (m_fontSize >= QUTE_XXLARGE)
+  if (m_fontSize >= QCS_XXLARGE)
     size = 7;
-  else if (m_fontSize >= QUTE_XLARGE)
+  else if (m_fontSize >= QCS_XLARGE)
     size = 6;
-  else if (m_fontSize >= QUTE_LARGE)
+  else if (m_fontSize >= QCS_LARGE)
     size = 5;
-  else if (m_fontSize >= QUTE_MEDIUM)
+  else if (m_fontSize >= QCS_MEDIUM)
     size = 4;
-  else if (m_fontSize >= QUTE_SMALL)
+  else if (m_fontSize >= QCS_SMALL)
     size = 3;
-  else if (m_fontSize >= QUTE_XSMALL)
+  else if (m_fontSize >= QCS_XSMALL)
     size = 2;
   else
     size = 1;
@@ -202,6 +211,53 @@ QString QuteText::getWidgetLine()
   return line;
 }
 
+QString QuteText::getWidgetType()
+{
+  return QString("BSBLabel");
+}
+
+QString QuteText::getWidgetXmlText()
+{
+  QXmlStreamWriter s(&xmlText);
+  createXmlWriter(s);
+
+  QString outText = m_text;
+  outText.replace(QRegExp("[\n\r]"), "\u00AC");
+  s.writeTextElement("label", outText);
+
+   //These are not implemented in blue
+  QString alignment;
+  if (((QLabel *)m_widget)->alignment() & Qt::AlignLeft)
+    alignment = "left";
+  else if (((QLabel *)m_widget)->alignment() & Qt::AlignCenter)
+    alignment = "center";
+  else if (((QLabel *)m_widget)->alignment() & Qt::AlignRight)
+    alignment = "right";
+  s.writeTextElement("alignment", alignment);
+  //TODO: what about type?
+
+  s.writeTextElement("font", m_font);
+  s.writeTextElement("fontsize", QString::number(m_fontSize));
+
+  QColor color = m_widget->palette().color(QPalette::WindowText);
+  s.writeStartElement("color");
+  s.writeTextElement("r", QString::number(color.red()));
+  s.writeTextElement("g", QString::number(color.green()));
+  s.writeTextElement("b", QString::number(color.blue()));
+  s.writeEndElement();
+  color = m_widget->palette().color(QPalette::Window);
+  s.writeStartElement("bgcolor");
+  s.writeAttribute("mode", m_widget->autoFillBackground()? "background":"nobackground");
+  s.writeTextElement("r", QString::number(color.red()));
+  s.writeTextElement("g", QString::number(color.green()));
+  s.writeTextElement("b", QString::number(color.blue()));
+  s.writeEndElement();
+  
+  s.writeTextElement("bordermode", static_cast<QFrame*>(m_widget)->frameShape()==QFrame::NoFrame ? "noborder": "border");
+  s.writeEndElement();
+  return xmlText;
+}
+
 void QuteText::createPropertiesDialog()
 {
   QuteWidget::createPropertiesDialog();
@@ -261,13 +317,13 @@ void QuteText::createPropertiesDialog()
   label->setText("Font Size");
   layout->addWidget(label, 8, 0, Qt::AlignRight|Qt::AlignVCenter);
   fontSize = new QComboBox(dialog);
-  fontSize->addItem("XX-Small", QVariant((int) QUTE_XXSMALL));
-  fontSize->addItem("X-Small", QVariant((int) QUTE_XSMALL));
-  fontSize->addItem("Small", QVariant((int) QUTE_SMALL));
-  fontSize->addItem("Medium", QVariant((int) QUTE_MEDIUM));
-  fontSize->addItem("Large", QVariant((int) QUTE_LARGE));
-  fontSize->addItem("X-Large", QVariant((int) QUTE_XLARGE));
-  fontSize->addItem("XX-Large", QVariant((int) QUTE_XXLARGE));
+  fontSize->addItem("XX-Small", QVariant((int) QCS_XXSMALL));
+  fontSize->addItem("X-Small", QVariant((int) QCS_XSMALL));
+  fontSize->addItem("Small", QVariant((int) QCS_SMALL));
+  fontSize->addItem("Medium", QVariant((int) QCS_MEDIUM));
+  fontSize->addItem("Large", QVariant((int) QCS_LARGE));
+  fontSize->addItem("X-Large", QVariant((int) QCS_XLARGE));
+  fontSize->addItem("XX-Large", QVariant((int) QCS_XXLARGE));
   int i = m_fontSize;
   int index = -1;
   while (i > 0 and index == -1) {
@@ -355,6 +411,11 @@ QuteLineEdit::~QuteLineEdit()
 {
 }
 
+void QuteLineEdit::loadFromXml(QString xmlText)
+{
+  qDebug() << "loadFromXml not implemented for this widget yet";
+}
+
 void QuteLineEdit::setAlignment(int alignment)
 {
 //   qDebug("QuteText::setAlignment %i", alignment);
@@ -412,6 +473,47 @@ QString QuteLineEdit::getWidgetLine()
   return line;
 }
 
+QString QuteLineEdit::getWidgetXmlText()
+{
+  QXmlStreamWriter s(&xmlText);
+  createXmlWriter(s);
+  // Not implemented in blue
+
+  s.writeTextElement("type", m_type);
+  s.writeTextElement("value", QString::number(static_cast<QDoubleSpinBox*>(m_widget)->value()));
+  s.writeTextElement("resolution", QString::number(m_resolution, 'f', 6));
+
+  s.writeTextElement("font", m_font);
+  s.writeTextElement("fontsize", QString::number(m_fontSize));
+
+  QColor color = m_widget->palette().color(QPalette::WindowText);
+  s.writeStartElement("color");
+  s.writeTextElement("r", QString::number(color.red()));
+  s.writeTextElement("g", QString::number(color.green()));
+  s.writeTextElement("b", QString::number(color.blue()));
+  s.writeEndElement();
+  color = m_widget->palette().color(QPalette::Window);
+  s.writeStartElement("bgcolor");
+  s.writeAttribute("mode", m_widget->autoFillBackground()? "background":"nobackground");
+  s.writeTextElement("r", QString::number(color.red()));
+  s.writeTextElement("g", QString::number(color.green()));
+  s.writeTextElement("b", QString::number(color.blue()));
+  s.writeEndElement();
+
+  s.writeTextElement("background", m_widget->autoFillBackground()? "background":"nobackground");
+  s.writeTextElement("border", "border");  //Not used yet. Should it be removed?
+
+  s.writeTextElement("randomizable", "");
+  s.writeEndElement();
+
+  return xmlText;
+}
+
+QString QuteLineEdit::getWidgetType()
+{
+  return QString("BSBLineEdit");
+}
+
 QString QuteLineEdit::getStringValue()
 {
   return static_cast<QLineEdit *>(m_widget)->text();
@@ -463,6 +565,11 @@ QuteScrollNumber::~QuteScrollNumber()
 {
 }
 
+void QuteScrollNumber::loadFromXml(QString xmlText)
+{
+  qDebug() << "loadFromXml not implemented for this widget yet";
+}
+
 void QuteScrollNumber::setResolution(double resolution)
 {
   m_resolution = resolution;
@@ -500,17 +607,17 @@ void QuteScrollNumber::setText(QString text)
 {
   m_text = text;
   int size;
-  if (m_fontSize >= QUTE_XXLARGE)
+  if (m_fontSize >= QCS_XXLARGE)
     size = 7;
-  else if (m_fontSize >= QUTE_XLARGE)
+  else if (m_fontSize >= QCS_XLARGE)
     size = 6;
-  else if (m_fontSize >= QUTE_LARGE)
+  else if (m_fontSize >= QCS_LARGE)
     size = 5;
-  else if (m_fontSize >= QUTE_MEDIUM)
+  else if (m_fontSize >= QCS_MEDIUM)
     size = 4;
-  else if (m_fontSize >= QUTE_SMALL)
+  else if (m_fontSize >= QCS_SMALL)
     size = 3;
-  else if (m_fontSize >= QUTE_XSMALL)
+  else if (m_fontSize >= QCS_XSMALL)
     size = 2;
   else
     size = 1;
@@ -554,6 +661,12 @@ QString QuteScrollNumber::getWidgetLine()
   return line;
 }
 
+QString QuteScrollNumber::getCabbageLine()
+{
+  QString line = "";
+  return line;
+}
+
 QString QuteScrollNumber::getCsladspaLine()
 {
   QString line = "ControlPort=" + m_name + "|" + m_name + "\n";
@@ -561,10 +674,14 @@ QString QuteScrollNumber::getCsladspaLine()
   return line;
 }
 
-QString QuteScrollNumber::getCabbageLine()
+QString QuteScrollNumber::getWidgetXmlText()
 {
-  QString line = "";
-  return line;
+  return QString();
+}
+
+QString QuteScrollNumber::getWidgetType()
+{
+  return QString("BSBScrollNumber");
 }
 
 QString QuteScrollNumber::getStringValue()
